//=============================================================================
// EriShop.js
//=============================================================================

/*:
 * @plugindesc Pivert Eri shop.
 * @author Pivert
 *
 * @param Gold variable
 * @desc Variable used to store gold
 * @default 1
 *
 * @param Watch variable
 * @desc Variable used to trigger watch event
 * @default 2
 *
 * @param First video ID
 * @desc ID of the first visible video (inclusive)
 * @default 100
 *
 * @param Last video ID
 * @desc ID of the last visible video (inclusive)
 * @default 199
 *
 * @param Video list offset
 * @desc Offset between first visible video and first invisible video
 * @default 200
 *
 * @param Multiplier
 * @desc Gold multiplier
 * @default 1000
 *
 */

(function() {
    'use strict';

    var parameters = PluginManager.parameters('EriShop');
    var goldVariable = Number(parameters['Gold variable']) || 1;
    var watchVariable = Number(parameters['Watch variable']) || 2;
    var objectListStart = Number(parameters['First video ID']) || 100;
    var objectListEnd = Number(parameters['Last video ID']) || 199;
    var objectListOffset = Number(parameters['Video list offset']) || 200;
    var multiplier = Number(parameters['Multiplier']) || 1000;

    //=============================================================================
    // Eri Shop Window Gold
    //=============================================================================
    function Window_EriShopGold() {
        this.initialize.apply(this, arguments);
    }

    Window_EriShopGold.prototype = Object.create(Window_Gold.prototype);
    Window_EriShopGold.prototype.constructor = Window_EriShopGold;

    Window_EriShopGold.prototype.initialize = function() {
        Window_Gold.prototype.initialize.call(this, Graphics.boxWidth - this.windowWidth(), 0);
    };

    Window_EriShopGold.prototype.windowWidth = function() {
        return 300;
    };

    Window_EriShopGold.prototype.drawCurrencyValue = function(value, unit, x, y, width) {
        value = $gameVariables.value(goldVariable) * multiplier;
        var unitWidth = Math.min(80, this.textWidth(unit));
        this.resetTextColor();
        this.drawText(value, x, y, width - unitWidth - 6, 'right');
        this.changeTextColor(this.systemColor());
        this.drawText(unit, x + width - unitWidth, y, unitWidth, 'right');
    };

    //=============================================================================
    // Eri Shop Window Sell
    //=============================================================================
    function Window_EriShopSell() {
        this.initialize.apply(this, arguments);
    }

    Window_EriShopSell.prototype = Object.create(Window_ItemList.prototype);
    Window_EriShopSell.prototype.constructor = Window_EriShopSell;

    Window_EriShopSell.prototype.initialize = function(x, y, width, height) {
        Window_ItemList.prototype.initialize.call(this, x, y, width, height);
    };

    Window_EriShopSell.prototype.drawItemNumber = function(item, x, y, width) {
        this.drawText(item.price * multiplier + ' ' + TextManager.currencyUnit, x, y, width, 'right');
    };

    Window_EriShopSell.prototype.includes = function(item) {
        return item && item.id >= objectListStart && item.id <= objectListEnd;
    };

    //=============================================================================
    // Eri Shop Scene Sell
    //=============================================================================
    function Scene_EriShopSell() {
        this.initialize.apply(this, arguments);
    }

    Scene_EriShopSell.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_EriShopSell.prototype.constructor = Scene_EriShopSell;

    Scene_EriShopSell.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_EriShopSell.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createGoldWindow();
        this.createTitleWindow();
        this.createSellWindow();
    };

    Scene_EriShopSell.prototype.createGoldWindow = function() {
        this._goldWindow = new Window_EriShopGold(0, 0);
        this.addWindow(this._goldWindow);
    };

    Scene_EriShopSell.prototype.createTitleWindow = function() {
        this._titleWindow = new Window_Help(1);
        this._titleWindow.width = Graphics.boxWidth - this._goldWindow.width;
        this._titleWindow.setText('What file do you want to upload?');
        this.addWindow(this._titleWindow);
    };

    Scene_EriShopSell.prototype.createSellWindow = function() {
        var wy = this._goldWindow.height;
        var wh = Graphics.boxHeight - wy;
        this._sellWindow = new Window_EriShopSell(0, wy, Graphics.boxWidth, wh);
        this._sellWindow.setHandler('ok', this.onSellOk.bind(this));
        this._sellWindow.setHandler('cancel', this.popScene.bind(this));
        this.addWindow(this._sellWindow);
        this._sellWindow.refresh();
        this._sellWindow.activate();
        this._sellWindow.select(0);
    };

    Scene_EriShopSell.prototype.onSellOk = function() {
        this._item = this._sellWindow.item();
        var gold = $gameVariables.value(goldVariable);
        $gameVariables.setValue(goldVariable, this._item.price + gold);
        $gameParty.loseItem(this._item, 1);
        var twin = $dataItems[this._item.id + objectListOffset];
        $gameParty.gainItem(twin, 1);
        var variable = +this._item.damage.formula;
        if (variable) {
            var value = $gameVariables.value(variable);
            $gameVariables.setValue(variable, value + 1);
        }
        this._goldWindow.refresh();
        this._sellWindow.refresh();
        this._sellWindow.activate();
        this._sellWindow.select(0);
    };

    //=============================================================================
    // Eri Shop Window Watch
    //=============================================================================
    function Window_EriShopWatch() {
        this.initialize.apply(this, arguments);
    }

    Window_EriShopWatch.prototype = Object.create(Window_ItemList.prototype);
    Window_EriShopWatch.prototype.constructor = Window_EriShopWatch;

    Window_EriShopWatch.prototype.initialize = function(x, y, width, height) {
        Window_ItemList.prototype.initialize.call(this, x, y, width, height);
    };

    Window_EriShopWatch._iconWidth = 128;
    Window_EriShopWatch._iconHeight = 96;

    Window_EriShopWatch.prototype.includes = function(item) {
        return item && item.id >= objectListStart + objectListOffset && item.id < objectListEnd + objectListOffset;
    };

    Window_EriShopWatch.prototype.isEnabled = function(item) {
        return true;
    };

    Window_EriShopWatch.prototype.itemHeight = function() {
        return 176;
    };

    Window_EriShopWatch.prototype.maxCols = function() {
        return 5;
    };

    Window_EriShopWatch.prototype.drawItem = function(index) {
        var item = this._data[index];
        if (item) {
            var rect = this.itemRect(index);
            this.changePaintOpacity(this.isEnabled(item));
            this.drawItemName(item, rect.x, rect.y + 4, rect.width);
        }
    };

    Window_EriShopWatch.prototype.drawItemName = function(item, x, y, width) {
        width = width || 312;
        if (item) {
            var iconBoxHeight = Window_EriShopWatch._iconHeight + 4;
            this.resetTextColor();
            this.drawIcon(item.iconIndex, x + (width - Window_EriShopWatch._iconWidth) / 2, y);
            var words = item.name.split(' ');
            var index = Math.round(words.length / 2);
            var firstLine = words.slice(0, index);
            var secondLine = words.slice(index);
            this.drawText(firstLine.join(' '), x, y + iconBoxHeight, width, 'center');
            this.drawText(secondLine.join(' '), x, y + iconBoxHeight + 32, width, 'center');
        }
    };

    Window_EriShopWatch.prototype.drawIcon = function(iconIndex, x, y) {
        var bitmap = ImageManager.loadSystem('IconSet');
        var pw = Window_EriShopWatch._iconWidth;
        var ph = Window_EriShopWatch._iconHeight;
        var sx = (iconIndex % 16) * Window_Base._iconWidth;
        var sy = Math.floor(iconIndex / 16) * Window_Base._iconHeight;
        this.contents.blt(bitmap, sx, sy, pw, ph, x, y);
    };

    //=============================================================================
    // Eri Shop Scene Watch
    //=============================================================================
    function Scene_EriShopWatch() {
        this.initialize.apply(this, arguments);
    }

    Scene_EriShopWatch.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_EriShopWatch.prototype.constructor = Scene_EriShopWatch;

    Scene_EriShopWatch.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_EriShopWatch.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createTitleWindow();
        this.createSellWindow();
    };

    Scene_EriShopWatch.prototype.createTitleWindow = function() {
        this._titleWindow = new Window_Help(1);
        this._titleWindow.setText('What file do you want to watch?');
        this.addWindow(this._titleWindow);
    };

    Scene_EriShopWatch.prototype.createSellWindow = function() {
        this._watchWindow = new Window_EriShopWatch(0, 0, Graphics.boxWidth, Graphics.boxHeight);
        this._watchWindow.setHandler('ok', this.onWatchOk.bind(this));
        this._watchWindow.setHandler('cancel', this.popScene.bind(this));
        this.addWindow(this._watchWindow);
        this._watchWindow.refresh();
        this._watchWindow.activate();
        this._watchWindow.select(0);
    };

    Scene_EriShopWatch.prototype.onWatchOk = function() {
        this._item = this._watchWindow.item();
        $gameVariables.setValue(watchVariable, this._item.id);
        SceneManager.pop();
    };

    //=============================================================================
    // Game Interpreter
    //=============================================================================
    var Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === 'EriShop' && args[0] === 'open') {
            SceneManager.push(Scene_EriShopSell);
        } else if (command === 'EriShop' && args[0] === 'watch') {
            SceneManager.push(Scene_EriShopWatch);
        }
    };
})();
